# -*- coding: UTF-8 -*-
import logging
from typing import List, Optional

import httpx
from dateutil.parser import parse

import qanty.common.models as models


logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger(__name__)


class Qanty:
    __url = "https://qanty.com/api"

    def __init__(self, auth_token: str, company_id: str) -> None:
        headers = {"Authorization": auth_token}
        self.client = httpx.Client(http2=True, headers=headers)
        self.company_id = company_id

    def __del__(self) -> None:
        self.client.close()

    def get_branches(self, filters: Optional[dict] = None, get_deleted: Optional[bool] = False) -> Optional[List[models.Branch]]:
        """
        Retrieves a list of branches for the company associated with this Qanty instance.

        :param filters: A dictionary of filters to apply to the branch list. Optional.
        :param get_deleted: Whether to include deleted branches in the list. Optional.
        :return: A list of Branch objects representing the branches for the company, or None if an error occurred.
        """
        url = f"{self.__url}/company/get_branches"
        try:
            response = self.client.post(url, json={"company_id": self.company_id, "filters": filters, "get_deleted": get_deleted})
            response.raise_for_status()
            data = response.json()
        except httpx.HTTPStatusError as exc:
            logger.error(exc)
            return None

        sites = data.get("sites", [])
        output: List[models.Branch] = [models.Branch.model_validate(item) for item in sites]
        return output

    def get_lines(
        self, branch_id: str, custom_branch_id: Optional[str] = None, get_deleted: Optional[bool] = False
    ) -> Optional[List[models.Line]]:
        """
        Retrieves a list of lines for a given branch ID.

        Args:
            branch_id (str): The ID of the branch to retrieve lines for.
            custom_branch_id (Optional[str], optional): The custom ID of the branch to retrieve lines for. Defaults to None.
            get_deleted (Optional[bool], optional): Whether to include deleted lines in the response. Defaults to False.

        Returns:
            Optional[List[models.Line]]: A list of Line objects representing the lines in the branch, or None if an error occurred.
        """
        url = f"{self.__url}/branches/get_lines"
        try:
            response = self.client.post(
                url,
                json={
                    "company_id": self.company_id,
                    "branch_id": branch_id,
                    "custom_branch_id": custom_branch_id,
                    "get_deleted": get_deleted,
                },
            )
            data = response.json()
        except httpx.HTTPStatusError as exc:
            logger.error(exc)
            return None

        if data.get("success") is False:
            logger.error(f"Error retrieving lines for branch {branch_id}: {data.get('msg')}")
            return None

        lines = data.get("lines", [])
        output: List[models.Line] = [models.Line.model_validate(item) for item in lines]
        return output

    def list_day_appointments_schedule(
        self, branch_id: str, line_id: str, day: str, custom_branch_id: Optional[str] = None
    ) -> Optional[List[models.AppointmentDaySchedule]]:
        """
        Retrieve the day schedule for a given branch and line on a specific day.

        Args:
            branch_id (str): The ID of the branch to retrieve the schedule for.
            line_id (str): The ID of the line to retrieve the schedule for.
            day (str): The day to retrieve the schedule for, in the format "YYYY-MM-DD".
            custom_branch_id (Optional[str], optional): The ID of a custom branch to retrieve the schedule for. Defaults to None.

        Returns:
            Optional[List[models.DaySchedule]]: A list of DaySchedule objects representing the appointments scheduled for the given branch and line on the given day, or None if an error occurred.
        """
        url = f"{self.__url}/appointments/list_day_schedule"
        try:
            response = self.client.post(
                url,
                json={
                    "company_id": self.company_id,
                    "branch_id": branch_id,
                    "line_id": line_id,
                    "day": day,
                    "custom_branch_id": custom_branch_id,
                },
            )
            data = response.json()
        except httpx.HTTPStatusError as exc:
            logger.error(exc)
            return None

        if data.get("success") is False:
            logger.error(f"Error retrieving day schedule for branch '{branch_id}' and line '{line_id}': {data.get('msg')}")
            return None

        appointments = data.get("appointments", {})

        output: List[models.AppointmentDaySchedule] = []
        for date, slots in appointments.items():
            entry = {"date_time": parse(date), "slots": []}
            for index, details in slots.items():
                entry["slots"].append({"index": index, **details})
                try:
                    output.append(models.AppointmentDaySchedule.model_validate(entry))
                except Exception:
                    logger.error(f"Error validating appointment day schedule entry: {entry}")
                    continue

        return output

    def list_day_orders_schedule(
        self, branch_id: str, line_id: str, day: str, custom_branch_id: Optional[str] = None
    ) -> Optional[List[models.AppointmentDaySchedule]]:
        pass



# -*- coding: UTF-8 -*-

import datetime

import pydantic
from typing import Any, Dict, List, Optional


class BranchLocation(pydantic.BaseModel):
    longitude: float
    latitude: float
    name: str
    address: str

    model_config = {"frozen": True}

    @classmethod
    @pydantic.field_validator("longitude")
    def validate_longitude(cls, value: float):
        if value < -180 or value > 180:
            raise ValueError("Longitude must be between -180 and 180")
        return value

    @classmethod
    @pydantic.field_validator("latitude")
    def validate_latitude(cls, value: float):
        if value < -90 or value > 90:
            raise ValueError("Latitude must be between -90 and 90")
        return value


class Branch(pydantic.BaseModel):
    id: str
    name: str
    gmt: str
    enabled: bool
    deleted: bool
    location: BranchLocation

    model_config = {"frozen": True}


class LineMetrics(pydantic.BaseModel):
    wait_avg: float
    wait_time: float
    min_wait_time: float
    max_wait_time: float
    serve_avg: float
    serve_time: float
    min_serve_time: float
    max_serve_time: float
    waiting_time_estimation: float

    model_config = {"frozen": True}


class AppointmentSettingsOperationRange(pydantic.BaseModel):
    howManyDays: int
    daysAhead: int


# TODO: This is incomplete
class AppointmentSettingsSetCalendars(pydantic.BaseModel):
    pass


class LineAppointmentSettings(pydantic.BaseModel):
    operationRange: AppointmentSettingsOperationRange
    setCalendars: List[AppointmentSettingsSetCalendars] = []
    cancelWindow: int
    sets: Dict[str, Any]
    enabled: bool


class Line(pydantic.BaseModel):
    id: str
    name: str
    chars: str
    waiting_tickets: int
    enabled: bool
    deleted: bool
    metrics: LineMetrics
    appointment_settings: LineAppointmentSettings
    color: Optional[str]

    model_config = {"frozen": True}


class AppointmentSlot(pydantic.BaseModel):
    index: str
    indexInsideSet: int
    duration: int
    appointment_slot_idx: int
    appointment_set: str
    attending_slot: str
    serve_as_group: bool
    status: str
    name: str


class AppointmentDaySchedule(pydantic.BaseModel):
    date_time: datetime.datetime
    slots: List[AppointmentSlot]

    model_config = {"frozen": True}



# -*- coding: UTF-8 -*-
import random

import qanty.common.models as models


def test_get_branch_lines(qanty):
    branches = qanty.get_branches()
    assert isinstance(branches, list)
    if len(branches) > 0:
        random_branch = random.choice(branches)
        lines = qanty.get_lines(branch_id=random_branch.id)
        assert isinstance(lines, list)
        if len(lines) > 0:
            for line in lines:
                assert isinstance(line, models.Line)


def test_get_deleted_lines(qanty):
    branches = qanty.get_branches()
    assert isinstance(branches, list)
    if len(branches) > 0:
        random_branch = random.choice(branches)
        lines = qanty.get_lines(branch_id=random_branch.id, get_deleted=True)
        assert isinstance(lines, list)
        if len(lines) > 0:
            for line in lines:
                assert isinstance(line, models.Line)


# def test_get_custom_branch_lines(qanty):
#     response = qanty.get_lines(custom_branch_id="custom_branch_id")
#     assert isinstance(response, list)


# def test_get_deleted_lines(qanty):
#     response = qanty.get_lines(get_deleted=True)
#     assert isinstance(response, list)



# -*- coding: UTF-8 -*-

import qanty.common.models as models


def test_get_branches(qanty):
    response = qanty.get_branches()
    assert isinstance(response, list)
    if len(response) > 0:
        for branch in response:
            assert isinstance(branch, models.Branch)


def test_get_deleted_branches(qanty):
    response = qanty.get_branches(get_deleted=True)
    assert isinstance(response, list)
    if len(response) > 0:
        for branch in response:
            assert isinstance(branch, models.Branch)


def test_get_branches_with_filters(qanty):
    response = qanty.get_branches(filters={"branch_groups": ["group1", "group2"]})
    assert isinstance(response, list)
    if len(response) > 0:
        for branch in response:
            assert isinstance(branch, models.Branch)



# -*- coding: UTF-8 -*-

import datetime

import qanty.common.models as models


def test_list_day_appointments_schedule(qanty):
    branches = qanty.get_branches()
    assert isinstance(branches, list)
    if len(branches) > 0:
        for branch in branches:
            lines = qanty.get_lines(branch_id=branch.id, get_deleted=True)
            assert isinstance(lines, list)
            if len(lines) == 0:
                continue

            for line in lines:
                if len(line.appointment_settings.sets) == 0:
                    continue

                appointments = qanty.list_day_appointments_schedule(
                    branch_id=branch.id,
                    line_id=line.id,
                    day=datetime.datetime.now().strftime("%Y-%m-%d"),
                )
                assert isinstance(appointments, list)

                break
            
            break



from setuptools import setup, find_packages

__version__ = "0.0.7"

setup(
    name="qanty",
    version=__version__,
    description="Qanty API Client",
    long_description=open("README.md").read().strip(),
    long_description_content_type="text/markdown",
    license="MIT",
    author="Juan F. Duque",
    author_email="jfelipe@grupodyd.com",
    packages=find_packages(exclude=["tests", "tests.*"]),
    include_package_data=True,
    url="https://www.qanty.com",
    project_urls={
        "Source": "https://github.com/grupodyd/python-qanty",
        "Tracker": "https://github.com/grupodyd/python-qanty/issues",
    },
    keywords="qanty",
    python_requires=">=3.8.0",
    install_requires=[
        "httpx[http2] >= 0.24.0, < 1",
        "pydantic >= 2, < 3",
        "python-dateutil >= 2.8.2, < 3",
    ],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
